function [Rt,St,Table]=cor2(r,B,A,R,S,Hr,Hs,Ts,tol,Fin,lam1,lam0)
% COR2 	is a Controller Order Reduction function based on CONID2 controller
%     	estimation method and using simulated data  
%			
%	[Rt,St,Table]=cor2(r,B,A,R,S,Hr,Hs,Ts,tol,Fin,lam1,lam0)
%
%	r is the excitation signal which is added to the output of the controller.
%	B and A are the numerator and denominator of the plant model.
%	R and S are the numerator and denominator of the initial controller.
%	Hr and Hs are the fixed terms on R and S (Robustness filter) with 
%  following default values: Hr=1,Hs=1 
%	Ts is the sampling period in Sec. (default=1)
%  tol is the tolerance value for vgap computing(default=0.001)
%  Fin is the initial gain (Fin=1000 by default)
%
% 		lam1 and lam0 make different adaptation algoritms as follows:
%		(lam1=0.95,lam0=1)
%
%		lam1=1;lam0=1   	 :decreasing gain 
%		0.95<lam1<1;lam0=1 :decreasing gain with fixed forgetting factor 
%		0.95<lam1,lam0<1	 :decreasing gain with variable forgetting factor 
%
%
%	Rt and St are the matrices containing the reduced order controllers.
%
%  Table is a matrix containing: 
%  -Controller No. (n) (zero corresponds to the initial controller).
%  -Orders of numerator (nR) and denominator (nS) of the reduced order 
%   controllers
%  -Vinnicombe's gap between initial and reducrd order controllers Vgap(R/S)
%  -Vinicombe's gap between the closed loop sensitivity functions of the 
%   initial closed loop and the closed loop system with the reduced order 
%   controllers Vgap(Sup),Vgap(Syp)
%  -Stability margin which is the inverse of the infinity norm of a matrix
%   containing four closed loop sensitivity functions (St-margin)
%  -Maximum of Syp together with fmax(Hz) for the closed loop system with 
%   reduced order controllers
%  -A stability indicatior which shows if the closed loop system is stable
%   (stable=1) or not (stable=0)
%
%  See also : COMPCON for comparing different reduced order controllers
%
% June 30,1999
% Copyright by Adaptech, 1997-1999
% Last modified: November 10, 2004

if nargin<5, error('[Rt,St,Table]=cor(r,B,A,R,S,Hr,Hs,Ts,tol,Fin,lam1,lam0)');end

if nargin<6, Hr=1;Hs=1;Ts=1;tol=0.001;Fin=1000;lam1=0.95;lam0=1;end
if nargin<7, Hs=1;Ts=1;tol=0.001;Fin=1000;lam1=0.95;lam0=1;end
if nargin<8, Ts=1;tol=0.001;Fin=1000;lam1=0.95;lam0=1;end
if nargin<9, tol=0.001;Fin=1000;lam1=0.95;lam0=1;end
if nargin<10, Fin=1000;lam1=0.95;lam0=1;end
if nargin<11, lam1=0.95;lam0=1;end
if nargin<12, lam0=1;end


if isempty(Hr), Hr=1;end
if isempty(Hs), Hs=1;end
if isempty(Ts), Ts=1;end
if isempty(tol), tol=0.001;end
if isempty(Fin), Fin=1000;end
if isempty(lam1), lam1=0.95;end
if isempty(lam0), lam0=1;end

P=addz(conv(A,S),conv(B,R));
u=filter(conv(B,R),P,r);

nR=length(R);
nS=length(S);

nHr=length(Hr);
nHs=length(Hs);
n1=nR-nHr;
n2=nS-nHs;
n=max(n1,n2);
w=logspace(-4,pi,round(1/tol));
z=exp(-j*w);
Table=[];

R1=R;S1=S;

cr=sprintf('\n');
disp([cr,' No.  nR  nS ','  Vg(R/S) ', ' Vg(Sup) ', ' Vg(Syp) ',...
      ' St-margin ', ' Max(Syp)[fmax] ',' stable'])
for i=n+1:-1:0,
   AS1=conv(A,S1);
   P1=addz(AS1,conv(B,R1));np1=length(P1);
   if max(abs(roots(P1)))<1, stab=1;else stab=0;end
   gap1=vgap(R1,S1,R,S);
   gap2=vgap(conv(A,R1),P1,conv(A,R),P);
   gap3=vgap(AS1,P1,conv(A,S),P);
   if stab==0, b=0; else b=smarg(B,A,R1,S1);end
   m=abs(polyval(AS1,z)./polyval(P1,z));
   [mdm,k]=max(20*log10(m));
   f=w(k)/(2*pi*Ts);
   data=sprintf('%3.0f%5.0f%4.0f%9.4f%9.4f%9.4f%10.4f%9.2f[%7.2f]%5.0f',...
      n-i+1,n1+nHr-1,n2+nHs-1,gap1,gap2,gap3,b,mdm,f,stab);
   disp(data)
   Table=[Table;n-i+1,n1+nHr-1,n2+nHs-1,gap1,gap2,gap3,b,mdm,f,stab];
   Rt(n-i+2,1:length(R1))=R1;
   St(n-i+2,1:length(S1))=S1;
   if i<n+1,
      if n1>0, n1=n1-1;end
      if n2>0, n2=n2-1;end
   end
   [R1,S1]=conid2(u,r,n1,n2,B,A,Hr,Hs,Fin,lam1,lam0);
end
cr=sprintf('\n');
disp(cr)